import os
import pathlib
from typing import Any, Generator, List, Optional, Tuple, TYPE_CHECKING
from unittest import mock

import pytest
from click.testing import CliRunner

from gcs_migration.commands.gcs_migration import main
from gcs_migration.models import MigrationPlan

if TYPE_CHECKING:
    from gcs_migration.typing.auth_documents import IdentityDocumentDict


@pytest.fixture
def mock_migration_plan(
    mock_get_transfer_api: Any,
    mock_get_auth_api: Any,
    migration_plan_cmd_create: MigrationPlan,
    authorization_response_success: Tuple[Optional[str], str, int],
    tmp_path: pathlib.Path,
) -> Generator[MigrationPlan, None, None]:
    with mock.patch.object(MigrationPlan, "load") as mock_migration_load:
        mock_migration_load.return_value = migration_plan_cmd_create
        assert migration_plan_cmd_create.storage_gateway is not None
        migration_plan_cmd_create.storage_gateway.allowed_domains = ["globus.org"]
        migration_plan_cmd_create.root = tmp_path
        os.mkdir(tmp_path / "migration_plan")
        yield migration_plan_cmd_create


@pytest.mark.parametrize(
    "globus_user,expect",
    [("user@globus.org", 0), ("unknownuser@globus.org", 1), ("user@unknown.org", 1)],
)
def test_set_guest_collection_owner(
    mock_migration_plan: MigrationPlan,
    share_endpoint_uuid: str,
    identity_document: "IdentityDocumentDict",
    globus_user: str,
    expect: int,
) -> None:
    runner = CliRunner()
    args: List[str] = [
        "set-guest-collection-owner",
        "-g",
        globus_user,
        share_endpoint_uuid,
    ]
    migration_plan = mock_migration_plan

    result = runner.invoke(main, args)
    assert result.exit_code == expect

    assert migration_plan.guest_collections is not None

    guest_updated = False
    for gc in migration_plan.guest_collections:
        if gc.id == share_endpoint_uuid:
            assert gc.credential is not None
            if expect == 0:
                assert gc.credential.identity_username == globus_user
                assert gc.credential.identity_id == identity_document["id"]
                guest_updated = True
            else:
                assert gc.credential.identity_username != globus_user
                assert gc.credential.identity_id != identity_document["id"]

    if expect == 0:
        assert guest_updated
    else:
        assert not guest_updated
